Imports Microsoft.Practices.Unity.InterceptionExtension
Imports System.Collections.Concurrent
Imports System.Reflection
Imports System.Threading.Tasks


' Microsoft patterns & practices   
' Unity Application Block   
' http://msdn.microsoft.com/unity
'===============================================================================   
' Copyright © Microsoft Corporation.  All rights reserved.   
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY   
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT   
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND   
' FITNESS FOR A PARTICULAR PURPOSE.   
'===============================================================================  

Namespace AsyncInterceptionSample
    Public Class LoggingAsynchronousOperationInterceptionBehavior
        Implements IInterceptionBehavior

        Private ReadOnly wrapperCreators As New ConcurrentDictionary(Of Type, Func(Of Task, IMethodInvocation, Task))

        Public Function Invoke(ByVal input As IMethodInvocation, ByVal getNext As GetNextInterceptionBehaviorDelegate) As IMethodReturn Implements IInterceptionBehavior.Invoke
            Dim value As IMethodReturn = getNext()(input, getNext)
            Dim method = TryCast(input.MethodBase, MethodInfo)

            If value.ReturnValue IsNot Nothing AndAlso method IsNot Nothing AndAlso GetType(Task).IsAssignableFrom(method.ReturnType) Then
                ' if this method returns a Task, override the original return value
                Dim task = CType(value.ReturnValue, Task)
                Return input.CreateMethodReturn(Me.GetWrapperCreator(method.ReturnType)(task, input), value.Outputs)
            End If

            Return value
        End Function

        Private Function GetWrapperCreator(ByVal taskType As Type) As Func(Of Task, IMethodInvocation, Task)
            Return Me.wrapperCreators.GetOrAdd(taskType, Function(t As Type)
                       If t Is GetType(Task) Then
                           Return AddressOf CreateWrapperTask
                       ElseIf t.IsGenericType AndAlso t.GetGenericTypeDefinition() Is GetType(Task(Of )) Then
                           Return CType(Me.GetType().GetMethod("CreateGenericWrapperTask", BindingFlags.Instance Or BindingFlags.NonPublic).MakeGenericMethod(New Type() {t.GenericTypeArguments(0)}).CreateDelegate(GetType(Func(Of Task, IMethodInvocation, Task)), Me), Func(Of Task, IMethodInvocation, Task))
                       Else
                           ' other cases are not supported
                           Return Function(task, IMethodInvocation) task
                       End If
                   End Function)
        End Function

        Private Async Function CreateWrapperTask(ByVal task As Task, ByVal input As IMethodInvocation) As Task
            Try
                Await task.ConfigureAwait(False)
                Trace.TraceInformation("Successfully finished async operation {0}", input.MethodBase.Name)
            Catch e As Exception
                Trace.TraceWarning("Async operation {0} threw: {1}", input.MethodBase.Name, e)
                Throw
            End Try
        End Function

        Private Function CreateGenericWrapperTask(Of T)(ByVal task As Task, ByVal input As IMethodInvocation) As Task
            Return Me.DoCreateGenericWrapperTask(Of T)(CType(task, Task(Of T)), input)
        End Function

        Private Async Function DoCreateGenericWrapperTask(Of T)(ByVal task As Task(Of T), ByVal input As IMethodInvocation) As Task(Of T)
            Try
                Dim value As T = Await task.ConfigureAwait(False)
                Trace.TraceInformation("Successfully finished async operation {0} with value: {1}", input.MethodBase.Name, value)
                Return value
            Catch e As Exception
                Trace.TraceWarning("Async operation {0} threw: {1}", input.MethodBase.Name, e)
                Throw
            End Try
        End Function

#Region "additional interception behavior methods"
        Public Function GetRequiredInterfaces() As IEnumerable(Of Type) Implements IInterceptionBehavior.GetRequiredInterfaces
            Return Type.EmptyTypes
        End Function

        Public ReadOnly Property WillExecute As Boolean Implements IInterceptionBehavior.WillExecute
            Get
                Return True
            End Get
        End Property
#End Region

    End Class
End Namespace
