Imports System.Threading.Tasks
Imports Microsoft.Practices.Unity.InterceptionExtension
Imports Microsoft.Practices.Unity

' Microsoft patterns & practices   
' Unity Application Block   
' http://msdn.microsoft.com/unity
'===============================================================================   
' Copyright © Microsoft Corporation.  All rights reserved.   
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY   
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT   
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND   
' FITNESS FOR A PARTICULAR PURPOSE.   
'===============================================================================  

Namespace AsyncInterceptionSample
    Friend Class Program
        Shared Sub Main(ByVal args() As String)
            Using container = New UnityContainer
                container.AddNewExtension(Of Interception)()
                container.RegisterType(Of ITestObject, TestObject)(New Interceptor(Of InterfaceInterceptor), New InterceptionBehavior(Of LoggingAsynchronousOperationInterceptionBehavior))

                Dim testObject = container.Resolve(Of ITestObject)()

                RunScenario("Application: Perform successful asynchronous operation", testObject, Sub()
                                                                                                      Console.WriteLine("Operation: doing something")
                                                                                                  End Sub)
                RunScenario("Application: Perform failed asynchronous operation", testObject, Sub()
                                                                                                  Console.WriteLine("Operation: about to fail")
                                                                                                  Throw New InvalidOperationException
                                                                                              End Sub)
                RunScenario("Application: Perform successful asynchronous operation with result", testObject, Function()
                                                                                                                  Console.WriteLine("Operation: doing something with int result")
                                                                                                                  Return 42
                                                                                                              End Function)
                RunScenario("Application: Perform successful asynchronous operation with result", testObject, Function()
                                                                                                                  Console.WriteLine("Operation: doing something with string result")
                                                                                                                  Return "test"
                                                                                                              End Function)
                RunScenario(Of Boolean)("Application: Perform failed asynchronous operation", testObject, Function()
                                                                                                              Console.WriteLine("Operation: about to fail")
                                                                                                              Throw New InvalidOperationException
                                                                                                          End Function)

                Console.WriteLine()
                Console.WriteLine("Press enter to end")
                Console.ReadLine()
            End Using
        End Sub

        Private Shared Sub RunScenario(ByVal title As String, ByVal testObject As ITestObject, ByVal action As Action)
            Try
                Console.WriteLine(title)
                testObject.DoSomethingAsync(action).Wait()
            Catch e As AggregateException
                Console.WriteLine("Application: caught exception {0}: {1}", e.InnerException.GetType.Name, e.InnerException.Message)
            End Try

            Console.WriteLine()
            Console.WriteLine("Press enter to continue")
            Console.ReadLine()
            Console.Clear()
        End Sub

        Private Shared Sub RunScenario(Of T)(ByVal title As String, ByVal testObject As ITestObject, ByVal action As Func(Of T))
            Try
                Console.WriteLine(title)
                Dim result As T = testObject.DoSomethingWithReturnValueAsync(action).Result
                Console.WriteLine("Application: received value {0}", result)
            Catch e As AggregateException
                Console.WriteLine("Application: caught exception {0}: {1}", e.InnerException.GetType.Name, e.InnerException.Message)
            End Try

            Console.WriteLine()
            Console.WriteLine("Press enter to continue")
            Console.ReadLine()
            Console.Clear()
        End Sub
    End Class

    Public Interface ITestObject
        Function DoSomethingAsync(ByVal someAction As Action) As Task
        Function DoSomethingWithReturnValueAsync(Of T)(ByVal someAction As Func(Of T)) As Task(Of T)
    End Interface

    Public Class TestObject
        Implements ITestObject

        Public Async Function DoSomethingWithReturnValueAsync(Of T)(ByVal someAction As Func(Of T)) As Task(Of T) Implements ITestObject.DoSomethingWithReturnValueAsync
            Await Task.Delay(750)
            Dim value = someAction()
            Await Task.Delay(750)
            Return value
        End Function

        Public Async Function DoSomethingAsync(ByVal someAction As Action) As Task Implements ITestObject.DoSomethingAsync
            Await Task.Delay(750)
            someAction()
            Await Task.Delay(750)
        End Function
    End Class
End Namespace
